"""
:mod:`zsl.tasks.asl.test_task`
------------------------------

Created on 22.12.2012

..moduleauthor:: Martin Babka <babka@atteq.com>
"""
from __future__ import absolute_import, division, print_function, unicode_literals

from builtins import *
import logging


class TestTask(object):
    def perform(self, _data):
        logging.getLogger(__name__).debug("Running zsl.tasks.zsl.TestTask")
        return "ok"



"""
:mod:`zsl.interface.webservice.performers.task`
-----------------------------------------------
"""
from __future__ import absolute_import, division, print_function, unicode_literals

from builtins import *  # NOQA
from functools import partial
import logging

from flask import request

from zsl import Injected, Zsl, inject
from zsl.application.error_handler import error_handler
from zsl.interface.web.utils.execution import convert_to_web_response, execute_web_task, notify_responders
from zsl.interface.web.utils.request_data import extract_data
from zsl.interface.web.utils.response_headers import append_headers
from zsl.router.task import TaskConfiguration, TaskRouter
from zsl.task.job_context import JobContext, WebJobContext


@inject(task_router=TaskRouter)
@append_headers
@notify_responders
@convert_to_web_response
@error_handler
def perform_web_task(namespace, path, task_router=Injected):
    logging.getLogger(__name__).debug("Performing task %s.", path)
    jc = WebJobContext(path, extract_data(request), None, None, request)
    (task, task_callable) = task_router.route(namespace + '/' + path)
    return execute_web_task(jc, lambda: task_callable(jc.task_data))


@inject(app=Zsl, task_configuration=TaskConfiguration)
def create_task_mapping(app, task_configuration):
    # type: (Zsl, TaskConfiguration)->None
    for namespace_configuration in task_configuration.namespaces:
        namespace = namespace_configuration.namespace.rstrip('/')
        f = partial(perform_web_task, namespace)
        f.__name__ = "perform-web-task-{0}".format(namespace)
        app.route("/{0}/<path:path>".format(namespace), methods=["POST", "GET", "OPTIONS"])(f)



"""
:mod:`zsl.tasks.asl.schedule_gearman_task`
------------------------------------------
"""
from __future__ import unicode_literals

from builtins import object

from zsl.interface.celery.worker import create_celery_app, zsl_task
from zsl.task.task_decorator import json_input


def schedule_celery_task(path, data):
    zsl_task.delay({'path': path, 'data': data})


class ScheduleCeleryTask(object):
    @json_input
    def perform(self, data):
        data = data.get_data()
        schedule_celery_task(data['path'], data['data'])
        return 'job submitted'



"""
:mod:`zsl.utils.injection_helper`
---------------------------------

.. moduleauthor:: Martin Babka
"""
from __future__ import absolute_import, division, print_function, unicode_literals

from builtins import *  # NOQA
import functools
import inspect
import logging
from typing import Type

from future.utils import viewitems
import injector
from injector import Binder, BindingKey, CallError, ClassProvider, Scope, reraise

from zsl._state import get_current_app


def instantiate(cls):
    inj = get_current_app().injector

    if hasattr(cls, "__new__"):
        task = inj.create_object(cls)
    else:
        task = cls()

    return task


def inject(**bindings):
    """
    Decorator for injecting parameters for ASL objects.
    """

    def outer_wrapper(f):
        def function_wrapper(ff):
            for key, value in viewitems(bindings):
                bindings[key] = BindingKey(value)

            @functools.wraps(ff)
            def _inject(*args, **kwargs):
                inj = get_current_app().injector
                dependencies = inj.args_to_inject(
                    function=ff,
                    bindings=bindings,
                    owner_key=ff
                )
                dependencies.update(kwargs)
                try:
                    return ff(*args, **dependencies)
                except TypeError as e:
                    reraise(e, CallError(ff, args, dependencies, e))

            return _inject

        '''
        Just a convenience method - delegates everything to wrapper.
        '''

        def method_or_class_wrapper(*a, **kwargs):
            """
            Properly installs the injector into the object so that the injection can be performed.
            """
            inj = get_current_app().injector
            # Install injector into the self instance if this is a method call.
            inj.install_into(a[0])

            # Use the generic wrapper.
            inject_f = injector.inject(**bindings)
            # Call the method.
            return inject_f(f)(*a, **kwargs)

        if inspect.ismethod(f):
            return method_or_class_wrapper
        else:
            return function_wrapper(f)

    return outer_wrapper


def bind(interface, to=None, scope=None):
    logging.getLogger(__name__).warning("Using this style of binding is deprecated. Use Modules instead.")
    get_current_app().injector.binder.bind(interface, to, scope)


def simple_bind(binder, cls, scope):
    # type: (Binder, Type, Scope) -> None
    binder.bind(interface=cls, to=ClassProvider(cls).get(binder.injector), scope=scope)



from __future__ import absolute_import, division, print_function, unicode_literals

from builtins import *


def extract_data(request):
    data = request.data
    json_data = request.get_json()
    return json_data if json_data else data
